package com.chelvc.framework.base.config;

import java.util.List;
import java.util.Map;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.DefaultSessionFactory;
import com.chelvc.framework.base.context.SessionFactory;
import com.chelvc.framework.base.interceptor.ResponseHandleInterceptor;
import com.chelvc.framework.base.interceptor.ResponseHandler;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Lists;
import org.springframework.beans.BeansException;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 上下文配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
public class ContextConfigurer implements ApplicationContextAware {
    /**
     * 初始化响应结果处理拦截器
     *
     * @param applicationContext 应用上下文
     */
    private void initializeResponseHandleInterceptor(ApplicationContext applicationContext) {
        RequestMappingHandlerAdapter adapter = applicationContext.getBean(RequestMappingHandlerAdapter.class);
        List<HandlerMethodReturnValueHandler> valueHandlers = adapter.getReturnValueHandlers();
        RequestResponseBodyMethodProcessor processor = CollectionUtils.isEmpty(valueHandlers) ? null :
                valueHandlers.stream().filter(handler -> handler instanceof RequestResponseBodyMethodProcessor)
                        .map(handler -> (RequestResponseBodyMethodProcessor) handler).findFirst().orElse(null);
        Map<String, ResponseHandler> beans = applicationContext.getBeansOfType(ResponseHandler.class);
        List<ResponseHandler> responseHandlers = ApplicationContextHolder.order(beans.values());
        ResponseHandleInterceptor interceptor = new ResponseHandleInterceptor(processor, responseHandlers);
        List<HandlerMethodReturnValueHandler> handlers = Lists.newLinkedList();
        handlers.add(interceptor);
        if (ObjectUtils.notEmpty(valueHandlers)) {
            handlers.addAll(valueHandlers);
        }
        adapter.setReturnValueHandlers(handlers);
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.initializeResponseHandleInterceptor(applicationContext);
    }

    @Bean
    @ConditionalOnMissingBean(SessionFactory.class)
    public SessionFactory sessionFactory() {
        return new DefaultSessionFactory();
    }
}
