package com.chelvc.framework.base.interceptor;

import java.util.List;

import com.google.common.collect.Lists;
import lombok.NonNull;
import org.springframework.core.MethodParameter;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.method.support.ModelAndViewContainer;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 响应结果处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class ResponseHandleInterceptor implements HandlerMethodReturnValueHandler {
    private final RequestResponseBodyMethodProcessor processor;
    private final List<ResponseHandler> handlers;

    public ResponseHandleInterceptor(@NonNull RequestResponseBodyMethodProcessor processor,
                                     @NonNull List<ResponseHandler> handlers) {
        this.processor = processor;
        this.handlers = Lists.newArrayList(handlers);
    }

    @Override
    public boolean supportsReturnType(MethodParameter method) {
        return true;
    }

    @Override
    public void handleReturnValue(Object value, MethodParameter method, ModelAndViewContainer container,
                                  NativeWebRequest request) throws Exception {
        for (ResponseHandler handler : this.handlers) {
            value = handler.handle(request, method, value);
        }
        this.processor.handleReturnValue(value, method, container, request);
    }
}
