package com.chelvc.framework.base.interceptor;

import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Result;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.NativeWebRequest;

/**
 * 响应结果包装处理器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Order
@Component
public class ResponseWrapHandler implements ResponseHandler {
    /**
     * 判断目标方法是否使用@ResponseBody注解
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isResponseBodyMethod(MethodParameter method) {
        return method.hasMethodAnnotation(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(ResponseBody.class)
                || method.getDeclaringClass().isAnnotationPresent(RestController.class);
    }

    /**
     * 判断目标方法是否使用了@ResponseWrapping注解
     *
     * @param method 目标方法
     * @return true/false
     */
    private boolean isResponseWrappingMethod(MethodParameter method) {
        ResponseWrapping annotation = method.getMethodAnnotation(ResponseWrapping.class);
        if (annotation == null) {
            annotation = method.getDeclaringClass().getAnnotation(ResponseWrapping.class);
        }
        return annotation != null && annotation.enabled();
    }

    @Override
    public Object handle(NativeWebRequest request, MethodParameter method, Object value) {
        if (!(value instanceof Result) && this.isResponseBodyMethod(method) && this.isResponseWrappingMethod(method)) {
            return Result.success(value, ApplicationContextHolder.getMessage("Success"));
        }
        return value;
    }
}
