package com.chelvc.framework.common.util;

import java.nio.charset.StandardCharsets;

import lombok.NonNull;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.codec.digest.MessageDigestAlgorithms;

/**
 * 编解码工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class CodecUtils {
    private CodecUtils() {
    }

    /**
     * MD5加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String md5(String plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.MD5);
    }

    /**
     * MD5加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static byte[] md5(byte[] plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.MD5);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static String sha256(String plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * SHA256加密
     *
     * @param plaintext 明文
     * @return 密文
     */
    public static byte[] sha256(byte[] plaintext) {
        return encrypt(plaintext, MessageDigestAlgorithms.SHA_256);
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static String encrypt(String plaintext, @NonNull String algorithm) {
        if (plaintext == null) {
            return null;
        }
        return Hex.encodeHexString(encrypt(plaintext.getBytes(StandardCharsets.UTF_8), algorithm));
    }

    /**
     * 数据加密
     *
     * @param plaintext 明文
     * @param algorithm 加密算法
     * @return 密文
     */
    public static byte[] encrypt(byte[] plaintext, @NonNull String algorithm) {
        return plaintext == null ? null : DigestUtils.getDigest(algorithm).digest(plaintext);
    }
}
