package com.chelvc.framework.database.handler;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import org.apache.ibatis.type.JdbcType;

/**
 * 敏感字符串列表字段处理实现
 *
 * @author Woody
 * @date 2024/6/17
 */
public class SensitiveStringsTypeHandler extends StringsTypeHandler {
    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, List<String> parameter, JdbcType jdbcType)
            throws SQLException {
        if (DatabaseContextHolder.isSensitiveWritable() && ObjectUtils.notEmpty(parameter)) {
            parameter = AESUtils.encrypt(DatabaseContextHolder.getEncryptor(), parameter);
        }
        super.setNonNullParameter(ps, i, parameter, jdbcType);
    }

    @Override
    public List<String> getNullableResult(ResultSet rs, String columnName) throws SQLException {
        List<String> values = super.getNullableResult(rs, columnName);
        if (ObjectUtils.isEmpty(values)) {
            return values;
        }
        return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
    }

    @Override
    public List<String> getNullableResult(ResultSet rs, int columnIndex) throws SQLException {
        List<String> values = super.getNullableResult(rs, columnIndex);
        if (ObjectUtils.isEmpty(values)) {
            return values;
        }
        return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
    }

    @Override
    public List<String> getNullableResult(CallableStatement cs, int columnIndex) throws SQLException {
        List<String> values = super.getNullableResult(cs, columnIndex);
        if (ObjectUtils.isEmpty(values)) {
            return values;
        }
        return AESUtils.decrypt(DatabaseContextHolder.getDecrypter(), values);
    }
}
