package com.chelvc.framework.feign.config;

import com.chelvc.framework.feign.interceptor.FeignFailureInterceptor;
import com.chelvc.framework.feign.interceptor.FeignInvokeInterceptor;
import feign.codec.Decoder;
import feign.codec.Encoder;
import feign.codec.ErrorDecoder;
import feign.optionals.OptionalDecoder;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.cloud.openfeign.FeignAutoConfiguration;
import org.springframework.cloud.openfeign.support.ResponseEntityDecoder;
import org.springframework.cloud.openfeign.support.SpringDecoder;
import org.springframework.cloud.openfeign.support.SpringEncoder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;

/**
 * Feign配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@AutoConfigureBefore(FeignAutoConfiguration.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class FeignConfigurer {
    private final ObjectFactory<HttpMessageConverters> messageConverters;

    @Bean
    @Primary
    @Scope(BeanDefinition.SCOPE_PROTOTYPE)
    public Encoder encoder() {
        return new CustomizeEncoder(new SpringEncoder(this.messageConverters));
    }

    @Bean
    @ConditionalOnMissingBean(Decoder.class)
    public Decoder decoder() {
        return new OptionalDecoder(new ResponseEntityDecoder(new SpringDecoder(this.messageConverters)));
    }

    @Bean
    @ConditionalOnMissingBean(ErrorDecoder.class)
    public ErrorDecoder errorDecoder() {
        return new FeignFailureInterceptor();
    }

    @Bean
    @ConditionalOnMissingBean(FeignInvokeInterceptor.class)
    public FeignInvokeInterceptor feignInvokeInterceptor() {
        return new FeignInvokeInterceptor();
    }
}
