package com.chelvc.framework.wechat.config;

import java.util.Collections;
import java.util.List;

import com.chelvc.framework.wechat.PayMode;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * 微信配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Data
@Configuration
@ConfigurationProperties("wechat")
public class WechatProperties {
    /**
     * 令牌配置
     */
    private final Token token = new Token();

    /**
     * 小程序配置列表
     */
    private List<Applet> applets = Collections.emptyList();

    /**
     * 公众号配置列表
     */
    private List<Public> publics = Collections.emptyList();

    /**
     * 支付配置列表
     */
    private List<Payment> payments = Collections.emptyList();

    /**
     * 令牌配置
     */
    @Data
    public static class Token {
        /**
         * 是否可刷新
         */
        private boolean refreshable;
    }

    /**
     * 小程序配置
     */
    @Data
    public static class Applet {
        /**
         * 唯一标识
         */
        private String id;

        /**
         * AppID
         */
        private String appid;

        /**
         * AppSecret
         */
        private String secret;
    }

    /**
     * 公众号配置
     */
    @Data
    public static class Public {
        /**
         * 唯一标识
         */
        private String id;

        /**
         * 数据密钥
         */
        private String key;

        /**
         * AppID
         */
        private String appid;

        /**
         * 公众号令牌
         */
        private String token;

        /**
         * AppSecret
         */
        private String secret;

        /**
         * 消息推送配置
         */
        private final Push push = new Push();
    }

    /**
     * 支付配置
     */
    @Data
    public static class Payment {
        /**
         * 应用分类
         */
        private String type;

        /**
         * 支付方式
         */
        private PayMode mode;

        /**
         * AppID
         */
        private String appid;

        /**
         * 商户标识
         */
        private String mchid;

        /**
         * 商户密钥
         */
        private String mchkey;

        /**
         * 微信回调地址
         */
        private String callback;
    }

    /**
     * 公众号消息推送配置
     */
    @Data
    public static class Push {
        /**
         * 小程序标识
         */
        private String applet;
    }
}
